
Elevator = class("Elevator")

Elevator.TWEEN_TIME = 1
Elevator.CAPACITY = 2
Elevator.HEIGHT = Floor.HEIGHT
Elevator.WIDTH = Elevator.CAPACITY * (Passenger.SIZE*2 + 4)

function Elevator:initialize(capacity, floor_number, floor_capacity)
  self.capacity = capacity
  self.floors = {}
  for i=1, floor_number do
    table.insert(self.floors, 1, Floor:new(floor_capacity, 0, Floor.HEIGHT*(i-1)))
  end
  self.current_floor = 0
  self.x = Floor.WIDTH
  self.y = (Floor.NUMBER - self.current_floor) * Floor.HEIGHT
  self.direction = 'up'
  self.passengers = {}
  self.enabled = false
  self.time = 4
end

function Elevator:update(dt)
	if self.enabled then
		self.time = self.time + dt
		if self.time >= Elevator.TWEEN_TIME then
			self.time = 0
			self.schedule(self)
		end
	end
	for i=1, #self.passengers do
    self.passengers[i]:update(dt)
  end
  for i=1, #self.floors do
  	self.floors[i]:update(dt)
  end
end

function Elevator:draw()
	love.graphics.rectangle(
			"line",
			math.floor(self.x)+0.5, math.floor(self.y)+0.5,
			Elevator.WIDTH, Elevator.HEIGHT)
  for i=1, #self.floors do
    self.floors[i]:draw()
  end
  for i=1, #self.passengers do
  	self.passengers[i]:draw()
  end
end

--This is the IO Scheduler algorithm
function Elevator:schedule()
	if self.direction == 'up' and self.current_floor == Floor.NUMBER then
		self.direction = 'down'
	elseif self.direction == 'down' and self.current_floor == 1 then
		self.direction = 'up'
	end
	if self.direction == 'up' then
		self.current_floor = self.current_floor + 1
	elseif self.direction == 'down' then
		self.current_floor = self.current_floor - 1
	end
	local passenger = self.floors[self.current_floor]:takePassenger()
	if passenger then
		table.insert(self.passengers, passenger)
		tween(
				Passenger.TWEEN_TIME,
				passenger,
				{x=self.x, y=passenger.y, alpha=0},
				'inOutCubic',
				table.remove(self.passengers, #self.passengers))
	end
	tween(
        Elevator.TWEEN_TIME,
        elevator,
        {y=(Floor.NUMBER - self.current_floor) * Floor.HEIGHT},
        'inOutCubic')
end
